@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -143,8 +143,6 @@ ieee80211_is_encrypted(tcpeditdlt_t *ctx, const void *packet, const int pktlen)
 u_char *
 ieee80211_get_src(const void *header)
 {
-    ieee80211_hdr_t *addr3;
-    ieee80211_addr4_hdr_t *addr4;
     uint16_t *frame_control, fc;
 
     assert(header);
@@ -152,10 +150,10 @@ ieee80211_get_src(const void *header)
     fc = ntohs(*frame_control);
 
     if (ieee80211_USE_4(fc)) {
-        addr4 = (ieee80211_addr4_hdr_t *)header;
+        ieee80211_addr4_hdr_t *addr4 = (ieee80211_addr4_hdr_t *)header;
         return addr4->addr4;
     } else {
-        addr3 = (ieee80211_hdr_t *)header;
+        ieee80211_hdr_t *addr3 = (ieee80211_hdr_t *)header;
         switch (fc & (ieee80211_FC_TO_DS_MASK + ieee80211_FC_FROM_DS_MASK)) {
             case ieee80211_FC_TO_DS_MASK:
                 return addr3->addr2;
@@ -173,8 +171,6 @@ ieee80211_get_src(const void *header)
 u_char *
 ieee80211_get_dst(const void *header)
 {
-    ieee80211_hdr_t *addr3;
-    ieee80211_addr4_hdr_t *addr4;
     uint16_t *frame_control, fc;
 
     assert(header);
@@ -182,10 +178,11 @@ ieee80211_get_dst(const void *header)
     fc = ntohs(*frame_control);
 
     if (ieee80211_USE_4(fc)) {
-        addr4 = (ieee80211_addr4_hdr_t *)header;
+        ieee80211_addr4_hdr_t *addr4 = (ieee80211_addr4_hdr_t *)header;
         return addr4->addr3;
     } else {
-        addr3 = (ieee80211_hdr_t *)header;
+        ieee80211_hdr_t *addr3 = (ieee80211_hdr_t *)header;
+
         switch (fc & (ieee80211_FC_TO_DS_MASK + ieee80211_FC_FROM_DS_MASK)) {
             case ieee80211_FC_TO_DS_MASK:
                 return addr3->addr3;
