@@ -47,7 +47,7 @@
  * packet or wait for a remote packet to arrive. 
 
  *
- * Usage: tcpliveplay <eth0/eth1> <file.pcap> <Destinatin IP [1.2.3.4]> <Destination mac [0a:1b:2c:3d:4e:5f]> <'random' dst port OR specify dport #>
+ * Usage: tcpliveplay <eth0/eth1> <file.pcap> <Destination IP [1.2.3.4]> <Destination mac [0a:1b:2c:3d:4e:5f]> <'random' dst port OR specify dport #>
  *
  * Example:  
  * yhsiam@yhsiam-VirtualBox:~$ tcpliveplay eth0 test1.pcap 192.168.1.4 52:57:01:11:31:92 random
@@ -164,7 +164,7 @@ main(int argc, char **argv)
 
     if((argc < 5) || (argv[1]==NULL) || (argv[2]==NULL) || (argv[3]==NULL) || (argv[4]==NULL) || (argv[5]==NULL)){
         printf("ERROR: Incorrect Usage!\n"); 
-        printf("Usage: tcpliveplay <eth0/eth1> <file.pcap> <Destinatin IP [1.2.3.4]> <Destination mac [0a:1b:2c:3d:4e:5f]> <specify 'random' or specific port#>\n");
+        printf("Usage: tcpliveplay <eth0/eth1> <file.pcap> <Destination IP [1.2.3.4]> <Destination mac [0a:1b:2c:3d:4e:5f]> <specify 'random' or specific port#>\n");
         printf("Example:\n    yhsiam@yhsiam-VirtualBox:~$ sudo tcpliveplay eth0 test1.pcap 192.168.1.4 52:57:01:11:31:92 random\n\n"); 
         exit(0);
     }
@@ -226,7 +226,7 @@ main(int argc, char **argv)
     }
 
     relative_sched(sched, sched[1].exp_rseq, num_packets);
-    printf("Packets Scheduled %d\n", pkts_scheduled);
+    printf("Packets Scheduled %u\n", pkts_scheduled);
 
     /* Open socket for savedfile traffic to be sent*/
     local_handle = pcap_open_offline("newfile.pcap", errbuf);   /*call pcap library function*/
@@ -259,7 +259,7 @@ main(int argc, char **argv)
     /* Start replay by sending the first packet, the SYN, from the schedule */
     else if(sched[0].local){  /* Send first packet*/
         sendpacket(sp, sched[sched_index].packet_ptr, sched[sched_index].pkthdr.len, &sched[sched_index].pkthdr);
-        printf("Sending Local Packet...............	[%d]\n",sched_index+1);
+        printf("Sending Local Packet...............	[%u]\n",sched_index+1);
         sched_index++; /* Proceed in the schedule */
     }
 
@@ -296,8 +296,8 @@ main(int argc, char **argv)
         } */
         /* Do the following if we receive a packet that ACKs for the same ACKing of next packet */
         else if((tcphdr_rprev->th_seq==htonl(sched[sched_index].exp_rseq)) && (tcphdr_rprev->th_ack==htonl(sched[sched_index].exp_rack)) && (size_payload_prev>0)){
-            printf("Received Remote Packet...............	[%d]\n",sched_index+1);
-            printf("Skipping Packet......................	[%d] to Packet [%d]\n",sched_index+1, sched_index+2);
+            printf("Received Remote Packet...............	[%u]\n",sched_index+1);
+            printf("Skipping Packet......................	[%u] to Packet [%u]\n",sched_index+1, sched_index+2);
             printf("Next Remote Packet Expectation met.\nProceeding in replay...\n");
             sched_index++;   
         } 
@@ -305,7 +305,7 @@ main(int argc, char **argv)
         else if(different_payload){
             printf("\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
             printf("+ WARNING: Remote host is not meeting packet size expectations.               +\n"); 
-            printf("+ for packet %-d. Application layer data differs from capture being replayed.  +\n", diff_payload_index+1);
+            printf("+ for packet %-u. Application layer data differs from capture being replayed.  +\n", diff_payload_index+1);
             printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n"); 
             printf("Requesting retransmission.\n Proceeding...\n");
             different_payload = false;  
@@ -315,7 +315,7 @@ main(int argc, char **argv)
         if(sched[sched_index].local) {
             /*Reset alarm timeout*/
             alarm (ALARM_TIMEOUT);
-            printf("Sending Local Packet...............	[%d]\n",sched_index+1); 
+            printf("Sending Local Packet...............	[%u]\n", sched_index + 1);
           
             /* edit each packet tcphdr before sending based on the schedule*/  
             if(sched_index>0){ 
@@ -326,7 +326,7 @@ main(int argc, char **argv)
             /* If 3 attempts of resending was made, then error out to the user */
             if(sched[sched_index].sent_counter==3){
                 printf("\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-                printf("+ ERROR: Re-sent packet [%-d] 3 times, but remote host is not  +\n", sched_index+1); 
+                printf("+ ERROR: Re-sent packet [%-u] 3 times, but remote host is not  +\n", sched_index+1);
                 printf("+ responding as expected. 3 resend attempts are a maximum.     +\n");
                 printf("+ Closing replay...                                            +\n");
                 printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n");
@@ -364,25 +364,26 @@ main(int argc, char **argv)
     /* User Debug Result Printouts*/
     if(sched_index==pkts_scheduled){
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"); 
-        printf("~ CONGRATS!!! You have successfully Replayed your pcap file '%s'  \n", argv[2]);
+        printf("~ CONGRATS!!! You have successfully Replayed your pcap file '%s'\n", argv[2]);
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
     }
     else {
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"); 
-        printf("~ Unfortunately an error has occurred  halting the replay of  \n");
-        printf("~ the pcap file '%s'. Please see error above for details...   \n", argv[2]);
+        printf("~ Unfortunately an error has occurred  halting the replay of\n");
+        printf("~ the pcap file '%s'. Please see error above for details...\n", argv[2]);
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
     }
 
     printf("----------------TCP Live Play Summary----------------\n");
-    printf("- Packets Scheduled to be Sent & Received:          %-d   \n", pkts_scheduled);
-    printf("- Actual Packets Sent & Received:                   %-d   \n", sched_index);
-    printf("- Total Local Packet Re-Transmissions due to packet       \n");
-    printf("- loss and/or differing payload size than expected: %-d   \n", retransmissions);
-    printf("- Thank you for Playing, Play again!                      \n");
+    printf("- Packets Scheduled to be Sent & Received:          %-u\n", pkts_scheduled);
+    printf("- Actual Packets Sent & Received:                   %-u\n", sched_index);
+    printf("- Total Local Packet Re-Transmissions due to packet\n");
+    printf("- loss and/or differing payload size than expected: %-u\n", retransmissions);
+    printf("- Thank you for Playing, Play again!\n");
     printf("----------------------------------------------------------\n\n");
 
     free(sched);
+    restore_stdin();
     return 0;
 } 
 /*end of main() function*/
@@ -448,7 +449,7 @@ relative_sched(struct tcp_sched* sched, u_int32_t first_rseq, int num_packets){
         else if(sched[i].remote){
             sched[i].exp_rseq = sched[i].exp_rseq - first_rseq; /* Fix expected remote SEQ to be relative */
             sched[i].exp_rack = sched[i].exp_rack - first_lseq; /* Fix expected remote ACK to be relative*/
-            sched[i].exp_rack = sched[i].exp_rack + lseq_adjust; /* Fix expeted remote ACK to be absolute */
+            sched[i].exp_rack = sched[i].exp_rack + lseq_adjust; /* Fix expected remote ACK to be absolute */
         }
     }
 
@@ -469,14 +470,15 @@ setup_sched(struct tcp_sched* sched){
 
     input_addr sip, dip;  /* Source & Destination IP */ 
     input_addr local_ip, remote_ip;    /* ip address of client and server*/
-    /*temporary packet buffers*/
-    struct pcap_pkthdr header;  // The header that pcap gives us
-    const u_char *packet;       // The actual packet
     pcap_t *local_handle;
+    const u_char *packet;       /* The actual packet */
+    unsigned int flags=0;
+    struct pcap_pkthdr header;  // The header that pcap gives us
     unsigned int pkt_counter=0;
-    ether_hdr *etherhdr = NULL;
-    tcp_hdr *tcphdr = NULL;
-    ipv4_hdr *iphdr = NULL;
+    bool remote = false;    /* flags to test if data is from 'client'=local or 'server'=remote */
+    bool local = false;
+    unsigned int i = 0;
+
     local_ip.byte1=0; 
     local_ip.byte2=0; 
     local_ip.byte3=0; 
@@ -487,13 +489,7 @@ setup_sched(struct tcp_sched* sched){
     remote_ip.byte3=0; 
     remote_ip.byte4=0; 
 
-    unsigned int size_ip, i = 0; 
-    unsigned int size_tcp; 
-    unsigned int size_payload; 
     char errbuf[PCAP_ERRBUF_SIZE];
-    unsigned int flags=0; 
-    bool remote = false;    /* flags to test if data is from 'cleint'=local or 'server'=remote */
-    bool local = false; 
 
 
     local_handle = pcap_open_offline("newfile.pcap", errbuf);   /*call pcap library function*/
@@ -504,12 +500,18 @@ setup_sched(struct tcp_sched* sched){
     }
 
     /*Before sending any packet, setup the schedule with the proper parameters*/
-    while((packet = pcap_next(local_handle,&header))) {
+    while((packet = safe_pcap_next(local_handle, &header))) {
+        /*temporary packet buffers*/
+        ether_hdr *etherhdr;
+        tcp_hdr *tcphdr;
+        ipv4_hdr *iphdr;
+        unsigned int size_ip;
+        unsigned int size_tcp;
+        unsigned int size_payload;
+
         pkt_counter++; /*increment number of packets seen*/
 
         memcpy(&sched[i].pkthdr, &header, sizeof(struct pcap_pkthdr));
-        //sched[i].len = header.len; 
-        //sched[i].caplen = header.caplen; 
         sched[i].packet_ptr = safe_malloc(sched[i].pkthdr.len);
         memcpy(sched[i].packet_ptr, packet, sched[i].pkthdr.len);
 
@@ -542,7 +544,7 @@ setup_sched(struct tcp_sched* sched){
             remote_ip = dip;
         }
 
-        /*Compare IPs to see which packet is this comming from*/
+        /*Compare IPs to see which packet is this coming from*/
         if(compip(&local_ip, &remote_ip, &sip)==LOCAL_IP_MATCH){
             local = true;
             remote = false;
@@ -610,7 +612,6 @@ setup_sched(struct tcp_sched* sched){
             //printf("REMOTE --------%d\n",i+1);
         }
 
-
         i++; /* increment schedule index */
 
     } /*end internal loop for reading packets (all in one file)*/
@@ -711,21 +712,17 @@ set_offline_filter(char* file)
  * This function is called every time we receive a remote packet
  */
 void 
-got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet){
-
-    ether_hdr *etherhdr = NULL; 
-    tcp_hdr *tcphdr = NULL;
-    ipv4_hdr *iphdr = NULL;
+got_packet(_U_ u_char *args, _U_ const struct pcap_pkthdr *header,
+        const u_char *packet)
+{
+    ether_hdr *etherhdr;
+    tcp_hdr *tcphdr;
+    ipv4_hdr *iphdr;
 
     unsigned int size_ip, size_tcp, size_payload;
     unsigned int flags = 0;
 
-    /* This is to get rid of the warning */
-    args = NULL;
-    if(args == NULL) header = NULL;
-    if(header == NULL) args = NULL; 
-
-    /* Extract and examine recieved packet headers */
+    /* Extract and examine received packet headers */
     etherhdr = (ether_hdr*)(packet);
     iphdr = (ipv4_hdr *)(packet + SIZE_ETHERNET);
     size_ip = iphdr->ip_hl << 2;
@@ -743,17 +740,17 @@ got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet)
 
 
     flags = tcphdr->th_flags;
-    /* Check correct SYN-ACK expecation, if so then proceed in fixing entire schedule from relative to absolute SEQs+ACKs */
+    /* Check correct SYN-ACK expectation, if so then proceed in fixing entire schedule from relative to absolute SEQs+ACKs */
     if((flags == (TH_SYN|TH_ACK)) && (sched_index==1) && (tcphdr->th_ack==htonl(sched[sched_index-1].curr_lseq + 1))){
         unsigned int j;
-        printf("Received Remote Packet...............	[%d]\n",sched_index+1);
+        printf("Received Remote Packet...............	[%u]\n", sched_index + 1);
         printf("Remote Pakcet Expectation met.\nProceeding in replay....\n");
         //printf("SYN-ACKed Random SEQ set!\n");
         initial_rseq = ntohl(tcphdr->th_seq); 
         //printf("initial_rseq: %u\n", initial_rseq);
         /* After we receiving the first SYN-ACK, then adjust the entire sched to be absolute rather than relative #s*/
         sched[1].exp_rseq = sched[1].exp_rseq + initial_rseq;
-        for(j = 2; j<pkts_scheduled; j++){ /* Based on correctly recieving the random SEQ from the SYN-ACK packet, do the following:*/
+        for(j = 2; j<pkts_scheduled; j++){ /* Based on correctly receiving the random SEQ from the SYN-ACK packet, do the following:*/
             if(sched[j].local){ /* Set local ACKs for entire sched to be absolute #s*/
                 sched[j].curr_lack = sched[j].curr_lack + initial_rseq; 
             }
@@ -821,7 +818,7 @@ got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet)
         acked_index = sched_index; /*Keep track correctly ACKed packet index*/
     } 
 
-    /* Global variable to keep tack of last recieved packet info */
+    /* Global variable to keep tack of last received packet info */
     packet_keeper_rprev = packet;  
     etherhdr_rprev = etherhdr; 
     tcphdr_rprev = tcphdr;
@@ -942,21 +939,14 @@ int
 rewrite(input_addr* new_remoteip, struct mac_addr* new_remotemac, input_addr* myip, struct mac_addr* mymac, char* file, unsigned int new_src_port)
 {
 
-    ether_hdr* etherhdr; 
-    ipv4_hdr *iphdr;
-    tcp_hdr *tcphdr;
+    char *newfile = "newfile.pcap";
+    int pkt_counter;
     input_addr local_ip; 
     input_addr remote_ip; 
-    unsigned int size_ip;
-    unsigned int size_tcp; 
-    char* newfile = "newfile.pcap";
-    char ErrBuff [1024];
-    int pkt_counter;
     const u_char *packet;
     struct pcap_pkthdr *header;
     pcap_dumper_t *dumpfile;
     input_addr sip;  /* Source IP */ 
-    unsigned int flags;
     int local_packets = 0;
     bool initstep1 = false;  /* keep track of successful handshake step */
     bool warned = false;
@@ -973,6 +963,7 @@ rewrite(input_addr* new_remoteip, struct mac_addr* new_remotemac, input_addr* my
 
     pcap_t *pcap = set_offline_filter(file); 
     if (!pcap){
+        char ErrBuff [1024];
         fprintf (stderr, "Cannot open PCAP file '%s' for reading\n", file);
         fprintf(stderr, "%s\n",ErrBuff);
         return PCAP_OPEN_ERROR;
@@ -985,7 +976,12 @@ rewrite(input_addr* new_remoteip, struct mac_addr* new_remotemac, input_addr* my
     }
 
     /*Modify each packet's IP & MAC based on the passed args then do a checksum of each packet*/
-    for (pkt_counter = 0; pcap_next_ex(pcap, &header, &packet) > 0; pkt_counter++){
+    for (pkt_counter = 0; safe_pcap_next_ex(pcap, &header, &packet) > 0; pkt_counter++){
+        unsigned int flags, size_ip;
+        ether_hdr* etherhdr;
+        ipv4_hdr *iphdr;
+        tcp_hdr *tcphdr;
+        unsigned int size_tcp;
 
         if (!warned && header->len > header->caplen) {
             fprintf(stderr, "warning: packet capture truncated to %d byte packets\n",
@@ -1164,17 +1160,17 @@ fix_all_checksum_liveplay(ipv4_hdr *iphdr){
 
 /************************************************************************************/
 
-/*[copied from Aaron Turnor's checksum.c, but ommitting tcpedit_t structs] */
+/*[copied from Aaron Turnor's checksum.c, but omitting tcpedit_t structs] */
 /*[The following functions have been slightly modified to be integrated with tcpliveplay code structure] */
 
 /** 
  * This code re-calcs the IP and Layer 4 checksums
  * the IMPORTANT THING is that the Layer 4 header 
- * is contiguious in memory after *ip_hdr we're actually
+ * is contiguous in memory after *ip_hdr we're actually
  * writing to the layer 4 header via the ip_hdr ptr.
  * (Yes, this sucks, but that's the way libnet works, and
  * I was too lazy to re-invent the wheel.
- * Returns 0 on sucess, -1 on error
+ * Returns 0 on success, -1 on error
  */
 
 
@@ -1186,9 +1182,8 @@ do_checksum_liveplay(u_int8_t *data, int proto, int len) {
     ipv4_hdr *ipv4;
     tcp_hdr *tcp;
     int ip_hl;
-    volatile int sum;   // <-- volatile works around a PPC g++ bug
+    volatile int sum = 0;   // <-- volatile works around a PPC g++ bug
 
-    sum;
     ipv4 = NULL;
 
     ipv4 = (ipv4_hdr *)data;
